/*******************************************************************************
 * 
 * jKiwi -- A virtual makeover and hairstyler application
 * Copyright (c) 2007-2008 Dan Mihai Ile, Maria Joao Roque Barbado Leal
 * 
 * This file is part of jKiwi.
 * 
 * jKiwi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * jKiwi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************/

package core;

import java.util.ResourceBundle;

import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;

import tabs.TabsContainer;
import utils.HelpWindow;
import utils.StockIcon;
import dialogs.AboutDialog;
import dialogs.ExampleImages;
import dialogs.PhotoBrowser;

/**
 * Instances of this class represents
 * the application's main window that
 * is used to create the image projects.
 */
public class MainWindow {

	/**
	 * The shell used by the main window
	 */
	private Shell sShell = null;

	//the menus in use by the main window
	private Menu menuBar = null;
	private Menu submenu = null;
	private Menu submenu1 = null;

	/**
	 * messages that are to be used by the entire GUI
	 */
	private static ResourceBundle messages;

	/**
	 * A container used as parent for the tabs
	 */
	private Composite mainContainer = null;

	/**
	 * A reference to this instance class
	 */
	private MainWindow mw = this;

//	unused in this version
//	private MirrorButton mirrorButton = null;
//	private Composite mirrorComposite;

	/**
	 * Key listener used to inform the project window
	 */
	private Listener sShellKeyListener;

	/**
	 * This method initializes mainContainer composite
	 */
	private void createMainContainer() {
		GridLayout gridLayout = new GridLayout();
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		RowData rowData = new org.eclipse.swt.layout.RowData(250, 580);
		mainContainer = new Composite(sShell, SWT.NONE);
		mainContainer.setLayoutData(rowData);
//		createMirrorButton();
		mainContainer.setLayout(gridLayout);
		mainContainer.setBackgroundImage(new Image(Display.getCurrent(), System.getProperty("program.pixmapspath") + "welcome.png"));

		//TODO drop should work when dropping directly from other program (firefox image)
		//TODO dispose the dt?
		DropTarget dt = new DropTarget(mainContainer, DND.DROP_COPY | DND.DROP_DEFAULT);
		dt.setTransfer(new Transfer[] {FileTransfer.getInstance()});
		dt.addDropListener(new DropTargetAdapter() {
			public void drop(DropTargetEvent event) {
				String[] strings = null;
				if (FileTransfer.getInstance().isSupportedType(event.currentDataType)) {
					strings = (String[]) event.data;
				}
				if (strings == null || strings.length == 0) { //!!Invalid data dropped
					return;
				}
				strings[0] = strings[0].replace("\n", ""); //for example Eye of GNOME puts an "\n" to the end of path..
				new ProjectWindow(mw, strings[0]);
			}

			public void dragEnter(DropTargetEvent event) {
				if (event.detail == DND.DROP_DEFAULT) {
					event.detail = DND.DROP_COPY;
				}
				sShell.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_BLACK));
			}

			public void dragOperationChanged(DropTargetEvent event) {
				if (event.detail == DND.DROP_DEFAULT) {
					event.detail = DND.DROP_COPY;
				}
			}

			public void dragLeave(DropTargetEvent event) {
				sShell.setBackground(null);
			}

			public void dropAccept(DropTargetEvent event) {
				sShell.setBackground(null);
			}
		});
	}

	/**
	 * Returns the main container composite
	 * which contains all the project's tabs
	 * @return the tabs container composite
	 */
	public Composite getMainContainer() {
		return mainContainer;
	}

	/**
	 * Returns the shell in use by this class
	 * @return the shell in use by this class
	 */
	public Shell getSShell() {
		return sShell;
	}

//	unused in this version
//	private void createMirrorButton() {
//	GridLayout gl = new GridLayout();
//	gl.marginHeight = 0;
//	gl.marginWidth = 0;
//	gl.horizontalSpacing = 0;
//	gl.verticalSpacing = 30;

//	GridData gridData = new GridData();
//	gridData.horizontalAlignment = org.eclipse.swt.layout.GridData.CENTER;
//	gridData.verticalAlignment = org.eclipse.swt.layout.GridData.CENTER;
//	gridData.grabExcessHorizontalSpace = true;
//	gridData.grabExcessVerticalSpace = true;
//	gridData.verticalIndent = 20;
//	mirrorComposite = new Composite(mainContainer, SWT.NONE);
//	mirrorComposite.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
//	mirrorComposite.setLayoutData(gridData);
//	mirrorComposite.setLayout(gl);
//	mirrorButton = new MirrorButton(mirrorComposite, SWT.NONE);
//	mirrorButton.setCursor(Display.getCurrent().getSystemCursor(SWT.CURSOR_HAND));
//	MirrorButton mirrorButton2 = new MirrorButton(mirrorComposite, SWT.NONE);
//	mirrorButton2.setCursor(Display.getCurrent().getSystemCursor(SWT.CURSOR_HAND));
//	}

	/**
	 * Creates a new instance of the MainWindow
	 */
	public MainWindow() {
		//set the locale and create the main window's shell
		messages = Loader.getMessages();
		createSShell();
	}

	/**
	 * This method initializes sShell
	 */
	private void createSShell() {

		RowLayout rowLayout = new RowLayout();
		rowLayout.justify = true;
		rowLayout.pack = false;
		sShell = new Shell(SWT.DIALOG_TRIM);
//		sShell = new Shell();
		sShell.setText(messages.getString("MAIN_WINDOW_TITLE"));
		sShell.setLayout(rowLayout);
		sShell.setImage(new Image(Display.getCurrent(), System.getProperty("program.icon")));
		createMainContainer();
//		sShell.setSize(new Point(100, 100));
		menuBar = new Menu(sShell, SWT.BAR);
		MenuItem submenuItem = new MenuItem(menuBar, SWT.CASCADE);
		submenuItem.setText(messages.getString("MAIN_WINDOW_MENU_FILE"));
		submenuItem.setSelection(false);
		MenuItem submenuItem1 = new MenuItem(menuBar, SWT.CASCADE);
		submenuItem1.setText(messages.getString("MAIN_WINDOW_MENU_HELP"));
		submenu1 = new Menu(submenuItem1);
		MenuItem push5 = new MenuItem(submenu1, SWT.PUSH);
		push5.setText(messages.getString("MAIN_WINDOW_MENU_HELP_HELP") + "\tCtrl+H");
		push5.setImage(new Image(Display.getCurrent(), StockIcon.getIcon(StockIcon.ICON_HELP, StockIcon.SIZE_MENU, true)));
		push5.setAccelerator(SWT.CTRL + 'H');
		push5.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				new HelpWindow(sShell).open();
			}
		});
		MenuItem push6 = new MenuItem(submenu1, SWT.PUSH);
		push6.setText(messages.getString("MAIN_WINDOW_MENU_HELP_ABOUT"));
		push6.setImage(new Image(Display.getCurrent(), StockIcon.getIcon(StockIcon.ICON_ABOUT, StockIcon.SIZE_MENU, true)));
		push6.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				new AboutDialog(sShell);
			}
		});
		submenuItem1.setMenu(submenu1);
		submenu = new Menu(submenuItem);
		MenuItem push1 = new MenuItem(submenu, SWT.PUSH);
		push1.setText(messages.getString("MAIN_WINDOW_MENU_FILE_OPEN") + "\tCtrl+O");
		push1.setAccelerator(SWT.CTRL + 'O');
		push1.setImage(new Image(Display.getCurrent(), StockIcon.getIcon(StockIcon.ICON_OPEN, StockIcon.SIZE_MENU, true)));
		push1.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				new ProjectWindow(mw, null);
			}
		});
		MenuItem push2 = new MenuItem(submenu, SWT.PUSH);
		push2.setText(messages.getString("MAIN_WINDOW_MENU_FILE_OPEN_EXAMPLE"));
		push2.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				String imagePath = new ExampleImages(mw.getSShell()).open();
				if (imagePath != null) {
					new ProjectWindow(mw, imagePath);
				}
			}
		});
		@SuppressWarnings("unused")
		MenuItem separator = new MenuItem(submenu, SWT.SEPARATOR);
		MenuItem push3 = new MenuItem(submenu, SWT.PUSH);
		push3.setText(messages.getString("MAIN_WINDOW_MENU_FILE_PREFERENCES"));
		push3.setImage(new Image(Display.getCurrent(), StockIcon.getIcon(StockIcon.ICON_PREFERENCES, StockIcon.SIZE_MENU, true)));
		push3.setEnabled(false);
		push3.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				System.out.println("widgetSelected()"); // TODO Auto-generated Event stub widgetSelected()
			}
		});
		@SuppressWarnings("unused")
		MenuItem separator1 = new MenuItem(submenu, SWT.SEPARATOR);
		MenuItem push4 = new MenuItem(submenu, SWT.PUSH);
		push4.setAccelerator(SWT.CTRL + 'Q');
		push4.setText(messages.getString("MAIN_WINDOW_MENU_FILE_QUIT") + "\tCtrl+Q");
		push4.setImage(new Image(Display.getCurrent(), StockIcon.getIcon(StockIcon.ICON_QUIT, StockIcon.SIZE_MENU, true)));
		push4.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				//TODO ask to save projects if required before closing
				sShell.close();
			}
		});
		submenuItem.setMenu(submenu);
		sShell.setMenuBar(menuBar);

		//if a key is pressed inform the active project window about it (if necessary)
		sShellKeyListener = new Listener() {
			public void handleEvent(Event e) {
				if (getCurrentTabsContainer() == null) return ;
				if (sShell.getDisplay().getActiveShell().equals(sShell)) { //certify that sShell is the active shell
					if (e.stateMask == SWT.CTRL && e.keyCode == 'z') {
						getCurrentTabsContainer().getParentProject().pushUndo(e);
					} else if (e.stateMask == SWT.CTRL && e.keyCode == 'y') {
						getCurrentTabsContainer().getParentProject().pushRedo(e);
					} else if (e.keyCode == SWT.F11) {
						getCurrentTabsContainer().getParentProject().pushFullScreen(e);
					} else if (e.stateMask == SWT.CTRL && e.keyCode == 'k') {
						getCurrentTabsContainer().getParentProject().pushClear(e);
					}
				}
			}
		};
		sShell.getDisplay().addFilter(SWT.KeyDown, sShellKeyListener);

		//sometimes the sShellKeyListener is not called because sShell has no focus but is active. this will fix it
		sShell.addShellListener(new ShellAdapter() {
			public void shellActivated(ShellEvent e) {
				sShell.setFocus();
				super.shellActivated(e);
				if (System.getProperty("import-photos") != null) {
					String file = new PhotoBrowser(sShell, System.getProperty("import-photos")).open();
					System.clearProperty("import-photos");
					if (file != null) {
						new ProjectWindow(mw, file);
					}
				}

			}
		});

		//remove the created listener when is no longer needed
		sShell.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				sShell.getDisplay().removeFilter(SWT.KeyDown, sShellKeyListener);
			}
		});
		sShell.pack();		
	}

	/**
	 * Returns the current visible TabsContainer or null if none is visible
	 * @return current TabsContainer
	 */
	private TabsContainer getCurrentTabsContainer() {
		for (Control c : getMainContainer().getChildren()) {
			if (c instanceof TabsContainer) {
				TabsContainer t = (TabsContainer) c;
				if (t.isVisible()) {
					return t;
				}
			}
		}
		return null;
	}

	/**
	 * inform this instance of MainWindow that a project TabsContainer was disposed
	 */
	public void tabDisposed() {
		boolean tabVisibleExists = false;
		for (Control c : getMainContainer().getChildren()) {
			if (c instanceof TabsContainer) {
				TabsContainer t = (TabsContainer) c;
				if (t.isVisible()) {
					tabVisibleExists = true;
					break;
				}
			}
		}
		if (!tabVisibleExists) {
//			mirrorComposite.setVisible(true);
		}
	}
}